/*
 * npshell.c
 *
 * Netscape Client Plugin API
 * - Functions that need to be implemented by plugin developers
 *
 * This file defines a "shell" plugin that plugin developers can use
 * as the basis for a real plugin.  This shell just provides empty
 * implementations of all functions that the plugin can implement
 * that will be called by Netscape (the NPP_xxx methods defined in 
 * npapi.h). 
 *
 *******************************************************************************
 * PLUGIN DEVELOPERS:
 *	Implement your plugins here.
 ******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <include/npapi.h>
#include <xp/npxp.h>		/* Express include file */

/*******************************************************************************
 * UNIX-only API calls
 ******************************************************************************/

#ifdef XP_UNIX

/* The defines PLUGIN_xxx can be found changed in npxp.h */

char*
NPP_GetMIMEDescription(void)
{
   return(PLUGIN_MIME_DESCRIPTION);
}

NPError
NPP_GetValue(void *future, 
	     NPPVariable variable, 
	     void *value)
{
   NPError err = NPERR_NO_ERROR;

   if (variable == NPPVpluginNameString)
      *((char **)value) = PLUGIN_NAME;
   else if (variable == NPPVpluginDescriptionString)
      *((char **)value) = PLUGIN_DESCRIPTION;
   else err = NPERR_GENERIC_ERROR;

   return err;
}

#endif 

/*******************************************************************************
 * General Plug-in Calls
 ******************************************************************************/

/*
** NPP_Initialize is called when your DLL is being loaded to do any
** DLL-specific initialization.
*/
NPError
NPP_Initialize(void)
{
  /*   MessageBox(NULL,"Loading AVS/Express\n",NULL,0); */
   return NPERR_NO_ERROR;
}

/*
** NPP_GetJavaClass is called during initialization to ask your plugin
** what its associated Java class is. If you don't have one, just return
** NULL. Otherwise, use the javah-generated "use_" function to both
** initialize your class and return it. If you can't find your class, an
** error will be signalled by "use_" and will cause the Navigator to
** complain to the user.
*/
jref
NPP_GetJavaClass(void)
{
   return NULL;            /* Java disabled */
}

/*
** NPP_Shutdown is called when your DLL is being unloaded to do any
** DLL-specific shut-down. You should be a good citizen and declare that
** you're not using your java class any more. This allows java to unload
** it, freeing up memory.
*/
void
NPP_Shutdown(void)
{   
/*    MessageBox(NULL,"Entering Shutdown\n",NULL,0); */

/*    MessageBox(NULL,"Leaving Shutdown\n",NULL,0); */
}

/*
** NPP_New is called when your plugin is instantiated (i.e. when an EMBED
** tag appears on a page).
*/
NPError 
NPP_New(NPMIMEType pluginType,
	NPP instance,
	uint16 mode,
	int16 argc,
	char* argn[],
	char* argv[],
	NPSavedData* saved)
{
   NPError result = NPERR_NO_ERROR;
   XPPluginInstance *This;


   if (instance == NULL)
      return NPERR_INVALID_INSTANCE_ERROR;
		
   instance->pdata = NPN_MemAlloc(sizeof(XPPluginInstance));
   This = (XPPluginInstance*)instance->pdata;
   if (This == NULL)
      return NPERR_OUT_OF_MEMORY_ERROR;
   else {
      /* mode is NP_EMBED, NP_FULL, or NP_BACKGROUND (see npapi.h) */
      This->fWindow = NULL;
      This->fMode = mode;
      This->valid_window = 0;

      This->window = 0;
      
      This->widget = 0;
      This->x = 0;
      This->y = 0;
      This->width = 0;
      This->height = 0;
      
      XP_New(This);

      /* PLUGIN DEVELOPERS:
       * Initialize fields of your plugin
       * instance data here.  If the NPSavedData is non-
       * NULL, you can use that data (returned by you from
       * NPP_Destroy to set up the new plugin instance).
      */
   }
   return result;
}


NPError 
NPP_Destroy(NPP instance, 
	    NPSavedData** save)
{
   XPPluginInstance *This;

   if (instance == NULL)
      return NPERR_INVALID_INSTANCE_ERROR;
   This = (XPPluginInstance*)instance->pdata;

   XP_Destroy(This); /* Call Bobs routine to destroy Express. */

   /* PLUGIN DEVELOPERS:
    * If desired, call NP_MemAlloc to create a
    * NPSavedDate structure containing any state information
    * that you want restored if this plugin instance is later
    * recreated.
   */
   if (This != NULL) {
      NPN_MemFree(instance->pdata);
      instance->pdata = NULL;
   }

   return NPERR_NO_ERROR;
}

NPError 
NPP_SetWindow(NPP instance, 
	      NPWindow* window)
{
   NPError result = NPERR_NO_ERROR;
   XPPluginInstance *This;
#ifdef XP_UNIX
  NPSetWindowCallbackStruct *ws_info = window->ws_info;
#endif

   if (instance == NULL)
      return NPERR_INVALID_INSTANCE_ERROR;
   This = (XPPluginInstance*)instance->pdata;

   This->fWindow = window;

   /*
    * PLUGIN DEVELOPERS:
    * Before setting window to point to the
    * new window, you may wish to compare the new window
    * info to the previous window (if any) to note window
    * size changes, etc.
   */

   /* If window becomes invalid - go and delete it. */
   if (window == NULL || window->window == NULL) {
      PLUGINDEBUGSTR("NPP_SetWindow: window not valid");
      This->valid_window = 0;
      XP_SetWindow(This);	/* delete window */
      return result;
   }
    
   /* If we have the same window, skip it - probably being resized */
#ifdef XP_UNIX
   if (This->valid_window && (This->window == (Window)window->window)) {
      PLUGINDEBUGSTR("NPP_SetWindow: same window");
      return result;
   }
   
#else
  if (This->valid_window && (This->window == (uint32)window->window)) {
    PLUGINDEBUGSTR("NPP_SetWindow: same window");
	return result;
	}
#endif

   PLUGINDEBUGSTR("NPP_SetWindow: new window");

   /* If we have a new window, but we've already created a viewer
      delete it before creating another one.
   */
   if (This->valid_window) {
      PLUGINDEBUGSTR("NPP_SetWindow: delete window");
      This->valid_window = 0;
      XP_SetWindow(This);	/* delete window */
   }    

   PLUGINDEBUGSTR("NPP_SetWindow: create window");

   
#ifdef XP_UNIX   
   This->widget = XtWindowToWidget(This->display, This->window);
   This->display = ws_info->display;	
#endif
   This->window = (int)window->window;
   This->x = window->x;
   This->y = window->y;
   This->width = window->width;
   This->height = window->height;
   This->valid_window = 1;
   PLUGINDEBUGSTR("NPP_SetWindow: made it this far \n");  

   XP_SetWindow(This);
	
   return result;
}

NPError 
NPP_NewStream(NPP instance,
	      NPMIMEType type,
	      NPStream *stream, 
	      NPBool seekable,
	      uint16 *stype)
{
   XPPluginInstance *This;

   if (instance == NULL)
      return NPERR_INVALID_INSTANCE_ERROR;
   This = (XPPluginInstance*)instance->pdata;

   return NPERR_NO_ERROR;
}


/* PLUGIN DEVELOPERS:
 * These next 2 functions are directly relevant in a plug-in which
 * handles the data in a streaming manner. If you want zero bytes
 * because no buffer space is YET available, return 0. As long as
 * the stream has not been written to the plugin, Navigator will
 * continue trying to send bytes.  If the plugin doesn't want them,
 * just return some large number from NPP_WriteReady(), and
 * ignore them in NPP_Write().  For a NP_ASFILE stream, they are
 * still called but can safely be ignored using this strategy.
 */

int32 STREAMBUFSIZE = 0X0FFFFFFF; 

/* If we are reading from a file in NPAsFile
 * mode so we can take any size stream in our
 * write call (since we ignore it) 
*/

int32 
NPP_WriteReady(NPP instance, 
	       NPStream *stream)
{
   XPPluginInstance *This;

   if (instance != NULL)
      This = (XPPluginInstance*)instance->pdata;

   /* Number of bytes ready to accept in NPP_Write() */
   return STREAMBUFSIZE;
}

int32 
NPP_Write(NPP instance, 
	  NPStream *stream, 
	  int32 offset, 
	  int32 len, 
	  void *buffer)
{
   XPPluginInstance *This;

   if (instance != NULL) 
      This = (XPPluginInstance*)instance->pdata;

   return len;		/* The number of bytes accepted */
}

NPError 
NPP_DestroyStream(NPP instance, NPStream *stream, NPError reason)
{
   XPPluginInstance *This;

   if (instance == NULL)
      return NPERR_INVALID_INSTANCE_ERROR;
   This = (XPPluginInstance*) instance->pdata;

   return NPERR_NO_ERROR;
}


void 
NPP_StreamAsFile(NPP instance, 
		 NPStream *stream, 
		 const char* fname)
{
   XPPluginInstance *This;

   if (instance != NULL)
      This = (XPPluginInstance*)instance->pdata;

}

void 
NPP_Print(NPP instance, 
	  NPPrint* printInfo)
{
   XPPluginInstance *This;

   if (printInfo == NULL)
      return;

   if (instance != NULL) {
      This = (XPPluginInstance*)instance->pdata;
	
      if (printInfo->mode == NP_FULL) {
         /* PLUGIN DEVELOPERS:
          * If your plugin would like to take over
          * printing completely when it is in full-screen mode,
          * set printInfo->pluginPrinted to TRUE and print your
          * plugin as you see fit.  If your plugin wants Netscape
          * to handle printing in this case, set
          * printInfo->pluginPrinted to FALSE (the default) and
          * do nothing.  If you do want to handle printing
          * yourself, printOne is true if the print button
          * (as opposed to the print menu) was clicked.
          * On the Macintosh, platformPrint is a THPrint; on
          * Windows, platformPrint is a structure
          * (defined in npapi.h) containing the printer name, port,
          * etc.
          */

         void* platformPrint = printInfo->print.fullPrint.platformPrint;
         NPBool printOne = printInfo->print.fullPrint.printOne;

         /* Do the default*/
         printInfo->print.fullPrint.pluginPrinted = FALSE;
      }
      else {	/* If not fullscreen, we must be embedded */
         /*
         * PLUGIN DEVELOPERS:
         * If your plugin is embedded, or is full-screen
         * but you returned false in pluginPrinted above, NPP_Print
         * will be called with mode == NP_EMBED.  The NPWindow
         * in the printInfo gives the location and dimensions of
         * the embedded plugin on the printed page.  On the
         * Macintosh, platformPrint is the printer port; on
         * Windows, platformPrint is the handle to the printing
         * device context.
         */

         NPWindow* printWindow = &(printInfo->print.embedPrint.window);
         void* platformPrint = printInfo->print.embedPrint.platformPrint;
      }
   }
}

/*******************************************************************************
// NPP_URLNotify:
// Notifies the instance of the completion of a URL request. 
// 
// NPP_URLNotify is called when Netscape completes a NPN_GetURLNotify or
// NPN_PostURLNotify request, to inform the plug-in that the request,
// identified by url, has completed for the reason specified by reason. The most
// common reason code is NPRES_DONE, indicating simply that the request
// completed normally. Other possible reason codes are NPRES_USER_BREAK,
// indicating that the request was halted due to a user action (for example,
// clicking the "Stop" button), and NPRES_NETWORK_ERR, indicating that the
// request could not be completed (for example, because the URL could not be
// found). The complete list of reason codes is found in npapi.h. 
// 
// The parameter notifyData is the same plug-in-private value passed as an
// argument to the corresponding NPN_GetURLNotify or NPN_PostURLNotify
// call, and can be used by your plug-in to uniquely identify the request. 
 ******************************************************************************/

void
NPP_URLNotify(NPP instance, 
	      const char* url, 
	      NPReason reason, 
	      void* notifyData)
{
}
